@@ -1,6 +1,6 @@
 from datetime import timedelta, datetime
 from functools import lru_cache
-from typing import Union, Generator
+from typing import Union, Generator, Tuple
 
 from numbers_parser.containers import ItemsList
 from numbers_parser.model import _NumbersModel
@@ -53,11 +53,18 @@ class Document:
         if table_name is None:
             table_name = "Table 1"
 
-        from_table_id = self._sheets[-1]._tables[0]._table_id
-        new_sheet_id = self._model.add_sheet(sheet_name, table_name, from_table_id)
-        self._sheets.append(Sheet(self._model, new_sheet_id))
+        prev_table_id = self._sheets[-1]._tables[0]._table_id
+        new_sheet_id = self._model.add_sheet(sheet_name)
+        new_sheet = Sheet(self._model, new_sheet_id)
+        new_sheet._tables.append(
+            Table(
+                self._model,
+                self._model.add_table(new_sheet_id, table_name, prev_table_id),
+            )
+        )
+        self._sheets.append(new_sheet)
 
-        return self._sheets[-1]
+        return new_sheet
 
 
 class Sheet:
@@ -73,15 +80,22 @@ class Sheet:
 
     @property
     def name(self):
+        """Return the sheets name"""
         return self._model.sheet_name(self._sheet_id)
 
     @name.setter
     def name(self, value):
+        """Set the sheet's name"""
         self._model.sheet_name(self._sheet_id, value)
 
-    def add_table(self, table_name=None, from_table_id=None) -> object:
+    def add_table(self, table_name=None, x=None, y=None) -> object:
         """Add a new table to the current sheet. If no sheet name is provided,
         the next available numbered sheet will be generated"""
+
+        from_table_id = self._tables[-1]._table_id
+        return self._add_table(table_name, from_table_id, x, y)
+
+    def _add_table(self, table_name, from_table_id, x, y) -> object:
         if table_name is not None:
             if table_name in self._tables:
                 raise IndexError(f"table '{table_name}' already exists")
@@ -91,7 +105,9 @@ class Sheet:
                 table_num += 1
             table_name = f"Table {table_num}"
 
-        new_table_id = self._model.add_table(self._sheet_id, table_name, from_table_id)
+        new_table_id = self._model.add_table(
+            self._sheet_id, table_name, from_table_id, x, y
+        )
         self._tables.append(Table(self._model, new_table_id))
         return self._tables[-1]
 
@@ -115,12 +131,24 @@ class Table:
 
     @property
     def name(self):
+        """Return the table's name"""
         return self._model.table_name(self._table_id)
 
     @name.setter
     def name(self, value):
+        """Set the table's name"""
         self._model.table_name(self._table_id, value)
 
+    @property
+    def height(self) -> float:
+        """Return the table's height in points"""
+        return self._model.table_height(self._table_id)
+
+    @property
+    def coordinates(self) -> Tuple[float]:
+        """Return the table's x,y offsets in points"""
+        return self._model.table_coordinates(self._table_id)
+
     @lru_cache(maxsize=None)
     def rows(self, values_only: bool = False) -> list:
         """
