@@ -6,16 +6,20 @@ from collections import OrderedDict
 from datetime import timedelta, datetime
 from functools import lru_cache
 from struct import pack, unpack
-from typing import Dict, List
+from typing import Dict, List, Tuple
 from uuid import uuid1
 from warnings import warn
 
 from numbers_parser.containers import ObjectStore
 from numbers_parser.constants import (
     EPOCH,
-    DEFAULT_DOCUMENT,
     DEFAULT_COLUMN_COUNT,
+    DEFAULT_COLUMN_WIDTH,
+    DEFAULT_DOCUMENT,
     DEFAULT_ROW_COUNT,
+    DEFAULT_ROW_HEIGHT,
+    DEFAULT_TABLE_OFFSET,
+    DEFAULT_TILE_SIZE,
     DOCUMENT_ID,
     PACKAGE_ID,
     MAX_TILE_SIZE,
@@ -125,8 +129,9 @@ class _NumbersModel:
         else:
             self.objects[sheet_id].name = value
 
-    # @lru_cache(maxsize=None)
-    def table_ids(self, sheet_id):
+    # Don't cache: new tables can be added at runtime
+    def table_ids(self, sheet_id: int) -> list:
+        """Return a list of table IDs for a given sheet ID"""
         table_info_ids = self.find_refs("TableInfoArchive")
         return [
             self.objects[t_id].tableModel.identifier
@@ -134,6 +139,16 @@ class _NumbersModel:
             if self.objects[t_id].super.parent.identifier == sheet_id
         ]
 
+    # Don't cache: new tables can be added at runtime
+    def table_info_id(self, table_id: int) -> int:
+        """Return the TableInfoArchive ID for a given table ID"""
+        ids = [
+            x
+            for x in self.objects.find_refs("TableInfoArchive")
+            if self.objects[x].tableModel.identifier == table_id
+        ]
+        return ids[0]
+
     @lru_cache(maxsize=None)
     def row_storage_map(self, table_id):
         # The base data store contains a reference to rowHeaders.buckets
@@ -181,7 +196,7 @@ class _NumbersModel:
 
     @lru_cache(maxsize=None)
     def table_format(self, table_id: int, key: int) -> str:
-        """Return the format assocuated with a format ID for a particular table"""
+        """Return the format associated with a format ID for a particular table"""
         bds = self.objects[table_id].base_data_store
         format_table_id = bds.format_table.identifier
         return self.table_format_entries(format_table_id)[key]
@@ -684,7 +699,7 @@ class _NumbersModel:
         )
         self.objects[PACKAGE_ID].components.append(component_info)
         self.add_component_reference(object_id, parent)
-        self.objects.mark_as_dirty(PACKAGE_ID)
+        self.objects.mark_as_dirty(object_id)
 
     def add_component_reference(
         self,
@@ -762,6 +777,7 @@ class _NumbersModel:
 
             tile_idx += 1
 
+        self.objects.mark_as_dirty(table_id)
         self.objects.update_dirty_objects()
 
     def create_string_table(self):
@@ -784,34 +800,48 @@ class _NumbersModel:
         }
         return style_sheet_map
 
-    def sheet_table_offset(self, sheet_id):
-        """Sum the heights of all rows in all tables in a sheet"""
-        if len(self.table_ids(sheet_id)) == 0:
-            return 0.0
+    def table_height(self, table_id: int) -> float:
+        """Return the height of a table in points"""
+        table_model = self.objects[table_id]
+        bds = self.objects[table_id].base_data_store
+        buckets = self.objects[bds.rowHeaders.buckets[0].identifier].headers
 
         height = 0.0
-        for table_id in self.table_ids(sheet_id):
-            table_model = self.objects[table_id]
-            # height = table_model.default_row_height * number_of_rows
-
-            bds = self.objects[table_id].base_data_store
-            buckets = self.objects[bds.rowHeaders.buckets[0].identifier].headers
-            for i, row in self.row_storage_map(table_id).items():
-                if row is not None:
-                    height += buckets[i].size
-                else:
-                    height += table_model.default_row_height
-
-        # TODO: calculate gap between tables
-        height += 100.0
-
+        for i, row in self.row_storage_map(table_id).items():
+            if row is not None and buckets[i].size != 0.0:
+                height += buckets[i].size
+            else:
+                height += table_model.default_row_height
         return height
 
+    def table_coordinates(self, table_id: int) -> Tuple[float]:
+        table_info = self.objects[self.table_info_id(table_id)]
+        return (
+            table_info.super.geometry.position.x,
+            table_info.super.geometry.position.y,
+        )
+
+    def last_table_offset(self, sheet_id):
+        """Y offset of the last table in a sheet"""
+        if len(self.table_ids(sheet_id)) == 0:
+            return 0.0
+        table_id = self.table_ids(sheet_id)[-1]
+        y_offset = [
+            self.objects[self.table_info_id(x)].super.geometry.position.y
+            for x in self.table_ids(sheet_id)
+            if x == table_id
+        ][0]
+
+        return self.table_height(table_id) + y_offset
+
     def add_table(
-        self, sheet_id: int, table_name: str, from_table_id: int = None
+        self,
+        sheet_id: int,
+        table_name: str,
+        from_table_id: int,
+        x: float = None,
+        y: float = None,
     ) -> int:
-        if from_table_id is None:
-            from_table_id = self.table_ids(sheet_id)[-1]
         from_table = self.objects[from_table_id]
 
         table_strings_id, table_strings = self.create_string_table()
@@ -825,8 +855,12 @@ class _NumbersModel:
                 "number_of_columns": DEFAULT_COLUMN_COUNT,
                 "table_name": table_name,
                 "table_name_enabled": True,
-                "default_row_height": 20.0,
-                "default_column_width": 98.0,
+                "default_row_height": DEFAULT_ROW_HEIGHT,
+                "default_column_width": DEFAULT_COLUMN_WIDTH,
+                "number_of_header_rows": 1,
+                "number_of_header_columns": 1,
+                "header_rows_frozen": True,
+                "header_columns_frozen": True,
                 **from_table_refs,
             },
             TSTArchives.TableModelArchive,
@@ -869,7 +903,7 @@ class _NumbersModel:
 
         self.add_component_metadata(
             format_table_pre_bnc_id,
-            "Document",
+            "CalculationEngine",
             "Tables/TableDataList-{}",
         )
 
@@ -888,7 +922,9 @@ class _NumbersModel:
                 nextColumnStripID=0,
                 rowTileTree=TSTArchives.TableRBTree(),
                 columnTileTree=TSTArchives.TableRBTree(),
-                tiles=TSTArchives.TileStorage(tile_size=256, should_use_wide_rows=True),
+                tiles=TSTArchives.TileStorage(
+                    tile_size=DEFAULT_TILE_SIZE, should_use_wide_rows=True
+                ),
                 **data_store_refs,
             )
         )
@@ -926,12 +962,23 @@ class _NumbersModel:
         table_info.tableModel.MergeFrom(
             TSPMessages.Reference(identifier=table_model_id)
         )
+
+        if x is not None:
+            table_x = x
+        else:
+            table_x = 0.0
+        if y is not None:
+            table_y = y
+        elif len(self.objects[sheet_id].drawable_infos) < 1:
+            table_y = 0.0
+        else:
+            table_y = self.last_table_offset(sheet_id) + DEFAULT_TABLE_OFFSET
         drawable = TSDArchives.DrawableArchive(
             parent=TSPMessages.Reference(identifier=sheet_id),
             geometry=TSDArchives.GeometryArchive(
                 angle=0.0,
                 flags=3,
-                position=TSPMessages.Point(x=0.0, y=self.sheet_table_offset(sheet_id)),
+                position=TSPMessages.Point(x=table_x, y=table_y),
                 size=TSPMessages.Size(height=231.0, width=494.0),
             ),
         )
@@ -943,20 +990,19 @@ class _NumbersModel:
 
         return table_model_id
 
-    def add_sheet(self, sheet_name: str, table_name: str, from_table_id: int):
+    def add_sheet(self, sheet_name: str):
         """Add a new sheet with a copy of a table from another sheet"""
-        sheet_id, sheet_archive = self.objects.create_object_from_dict(
+        sheet_id, _ = self.objects.create_object_from_dict(
             "Document", {"name": sheet_name}, TNArchives.SheetArchive
         )
 
-        table_info_id = self.add_table(sheet_id, table_name, from_table_id)
+        self.add_component_reference(
+            sheet_id, "CalculationEngine", DOCUMENT_ID, is_weak=True
+        )
 
         self.objects[DOCUMENT_ID].sheets.append(
             TSPMessages.Reference(identifier=sheet_id)
         )
-        sheet_archive.drawable_infos.append(
-            TSPMessages.Reference(identifier=table_info_id)
-        )
 
         return sheet_id
 
